@@ -17,7 +17,7 @@ import os.path as op
 import numpy as np
 
 from ..transforms import _pol_to_cart, _cart_to_sph
-from ..io.pick import pick_types, _picks_to_idx
+from ..io.pick import pick_types, _picks_to_idx, _FNIRS_CH_TYPES_SPLIT
 from ..io.constants import FIFF
 from ..io.meas_info import Info
 from ..utils import (_clean_names, warn, _check_ch_locs, fill_doc,
@@ -36,7 +36,8 @@ class Layout(object):
     box : tuple of length 4
         The box dimension (x_min, x_max, y_min, y_max).
     pos : array, shape=(n_channels, 4)
-        The positions of the channels in 2d (x, y, width, height).
+        The unit-normalized positions of the channels in 2d
+        (x, y, width, height).
     names : list
         The channel names.
     ids : list
@@ -91,12 +92,14 @@ class Layout(object):
                                                      ', '.join(self.names[:3]))
 
     @fill_doc
-    def plot(self, picks=None, show=True):
+    def plot(self, picks=None, show_axes=False, show=True):
         """Plot the sensor positions.
 
         Parameters
         ----------
         %(picks_nostr)s
+        show_axes : bool
+            Show layout axes if True. Defaults to False.
         show : bool
             Show figure if True. Defaults to True.
 
@@ -110,7 +113,7 @@ class Layout(object):
         .. versionadded:: 0.12.0
         """
         from ..viz.topomap import plot_layout
-        return plot_layout(self, picks=picks, show=show)
+        return plot_layout(self, picks=picks, show_axes=show_axes, show=show)
 
 
 def _read_lout(fname):
@@ -214,14 +217,14 @@ def read_layout(kind, path=None, scale=True):
     return Layout(box=box, pos=pos, names=names, kind=kind, ids=ids)
 
 
+@fill_doc
 def make_eeg_layout(info, radius=0.5, width=None, height=None, exclude='bads',
                     csd=False):
     """Create .lout file from EEG electrode digitization.
 
     Parameters
     ----------
-    info : instance of Info
-        Measurement info (e.g., raw.info).
+    %(info_not_none)s
     radius : float
         Viewport radius as a fraction of main figure height. Defaults to 0.5.
     width : float | None
@@ -300,9 +303,7 @@ def make_grid_layout(info, picks=None, n_col=None):
 
     Parameters
     ----------
-    info : instance of Info | None
-        Measurement info (e.g., raw.info). If None, default names will be
-        employed.
+    %(info_not_none)s
     %(picks_base)s all good misc channels.
     n_col : int | None
         Number of columns to generate. If None, a square grid will be produced.
@@ -367,13 +368,13 @@ def make_grid_layout(info, picks=None, n_col=None):
     return layout
 
 
+@fill_doc
 def find_layout(info, ch_type=None, exclude='bads'):
     """Choose a layout based on the channels in the info 'chs' field.
 
     Parameters
     ----------
-    info : instance of Info
-        The measurement info.
+    %(info_not_none)s
     ch_type : {'mag', 'grad', 'meg', 'eeg'} | None
         The channel type for selecting single channel layouts.
         Defaults to None. Note, this argument will only be considered for
@@ -457,13 +458,13 @@ def find_layout(info, ch_type=None, exclude='bads'):
     return layout
 
 
+@fill_doc
 def _find_kit_layout(info, n_grads):
     """Determine the KIT layout.
 
     Parameters
     ----------
-    info : Info
-        Info object.
+    %(info_not_none)s
     n_grads : int
         Number of KIT-gradiometers in the info.
 
@@ -592,14 +593,14 @@ def _box_size(points, width=None, height=None, padding=0.0):
     return width, height
 
 
+@fill_doc
 def _find_topomap_coords(info, picks, layout=None, ignore_overlap=False,
                          to_sphere=True, sphere=None):
     """Guess the E/MEG layout and return appropriate topomap coordinates.
 
     Parameters
     ----------
-    info : instance of Info
-        Measurement info.
+    %(info_not_none)s
     picks : str | list | slice | None
         None will choose all channels.
     layout : None | instance of Layout
@@ -628,6 +629,7 @@ def _find_topomap_coords(info, picks, layout=None, ignore_overlap=False,
     return pos
 
 
+@fill_doc
 def _auto_topomap_coords(info, picks, ignore_overlap, to_sphere, sphere):
     """Make a 2 dimensional sensor map from sensor positions in an info dict.
 
@@ -637,8 +639,7 @@ def _auto_topomap_coords(info, picks, ignore_overlap, to_sphere, sphere):
 
     Parameters
     ----------
-    info : instance of Info
-        The measurement info.
+    %(info_not_none)s
     picks : list | str | slice | None
         None will pick all channels.
     ignore_overlap : bool
@@ -666,7 +667,7 @@ def _auto_topomap_coords(info, picks, ignore_overlap, to_sphere, sphere):
     locs3d = np.array([ch['loc'][:3] for ch in chs])
 
     # If electrode locations are not available, use digization points
-    if not _check_ch_locs(chs):
+    if not _check_ch_locs(info=info, picks=picks):
         logging.warning('Did not find any electrode locations (in the info '
                         'object), will attempt to use digitization points '
                         'instead. However, if digitization points do not '
@@ -771,14 +772,14 @@ def _topo_to_sphere(pos, eegs):
     return np.column_stack([xs, ys, zs])
 
 
+@fill_doc
 def _pair_grad_sensors(info, layout=None, topomap_coords=True, exclude='bads',
                        raise_error=True):
     """Find the picks for pairing grad channels.
 
     Parameters
     ----------
-    info : instance of Info
-        An info dictionary containing channel information.
+    %(info_not_none)s
     layout : Layout | None
         The layout if available. Defaults to None.
     topomap_coords : bool
@@ -917,7 +918,7 @@ def _merge_ch_data(data, ch_type, names, method='rms'):
     if ch_type == 'grad':
         data = _merge_grad_data(data, method)
     else:
-        assert ch_type in ('hbo', 'hbr', 'fnirs_cw_amplitude', 'fnirs_od')
+        assert ch_type in _FNIRS_CH_TYPES_SPLIT
         data, names = _merge_nirs_data(data, names)
     return data, names
 
