@@ -1,13 +1,13 @@
 # Author : Martin Luessi mluessi@nmr.mgh.harvard.edu (2012)
-# License : BSD 3-clause
+# License : BSD-3-Clause
 
 # Parts of this code were copied from NiTime http://nipy.sourceforge.net/nitime
 
 import operator
 import numpy as np
 
-from ..fixes import _get_dpss, rfft, irfft, rfftfreq
-from ..parallel import parallel_func
+from ..fixes import _import_fft
+from ..parallel import parallel_func, check_n_jobs
 from ..utils import sum_squared, warn, verbose, logger, _check_option
 
 
@@ -53,14 +53,16 @@ def dpss_windows(N, half_nbw, Kmax, low_bias=True, interp_from=None,
 
     Notes
     -----
-    Tridiagonal form of DPSS calculation from:
+    Tridiagonal form of DPSS calculation from :footcite:`Slepian1978`.
 
-    Slepian, D. Prolate spheroidal wave functions, Fourier analysis, and
-    uncertainty V: The discrete case. Bell System Technical Journal,
-    Volume 57 (1978), 1371430
+    References
+    ----------
+    .. footbibliography::
     """
     from scipy import interpolate
+    from scipy.signal.windows import dpss as sp_dpss
     from ..filter import next_fast_len
+    rfft, irfft = _import_fft(('rfft', 'irfft'))
     # This np.int32 business works around a weird Windows bug, see
     # gh-5039 and https://github.com/scipy/scipy/pull/8608
     Kmax = np.int32(operator.index(Kmax))
@@ -92,7 +94,7 @@ def dpss_windows(N, half_nbw, Kmax, low_bias=True, interp_from=None,
         dpss = np.array(dpss)
 
     else:
-        dpss = _get_dpss()(N, half_nbw, Kmax)
+        dpss = sp_dpss(N, half_nbw, Kmax)
 
     # Now find the eigenvalues of the original spectral concentration problem
     # Use the autocorr sequence technique from Percival and Walden, 1993 pg 390
@@ -298,6 +300,7 @@ def _mt_spectra(x, dpss, sfreq, n_fft=None):
     freqs : array
         The frequency points in Hz of the spectra
     """
+    rfft, rfftfreq = _import_fft(('rfft', 'rfftfreq'))
     if n_fft is None:
         n_fft = x.shape[-1]
 
@@ -409,6 +412,7 @@ def psd_array_multitaper(x, sfreq, fmin=0, fmax=np.inf, bandwidth=None,
     -----
     .. versionadded:: 0.14.0
     """
+    rfftfreq = _import_fft('rfftfreq')
     _check_option('normalization', normalization, ['length', 'full'])
 
     # Reshape data so its 2-D for parallelization
@@ -428,6 +432,7 @@ def psd_array_multitaper(x, sfreq, fmin=0, fmax=np.inf, bandwidth=None,
 
     psd = np.zeros((x.shape[0], freq_mask.sum()))
     # Let's go in up to 50 MB chunks of signals to save memory
+    n_jobs = check_n_jobs(n_jobs)
     n_chunk = max(50000000 // (len(freq_mask) * len(eigvals) * 16), n_jobs)
     offsets = np.concatenate((np.arange(0, x.shape[0], n_chunk), [x.shape[0]]))
     for start, stop in zip(offsets[:-1], offsets[1:]):
