@@ -69,7 +69,8 @@ def set_memmap_min_size(memmap_min_size):
 known_config_types = (
     'MNE_3D_OPTION_ANTIALIAS',
     'MNE_BROWSE_RAW_SIZE',
-    'MNE_BROWSE_BACKEND',
+    'MNE_BROWSER_BACKEND',
+    'MNE_BROWSER_USE_OPENGL',
     'MNE_CACHE_DIR',
     'MNE_COREG_ADVANCED_RENDERING',
     'MNE_COREG_COPY_ANNOT',
@@ -473,9 +474,9 @@ def sys_info(fid=None, show_paths=False, *, dependencies='user'):
         Can be None to use :data:`sys.stdout`.
     show_paths : bool
         If True, print paths for each module.
-    dependencies : str
-        Can be "user" (default) to show user-relevant dependencies, or
-        "developer" to additionally show developer dependencies.
+    dependencies : 'user' | 'developer'
+        Show dependencies relevant for users (default) or for developers
+        (i.e., output includes additional dependencies).
 
         .. versionadded:: 0.24
 
@@ -548,7 +549,7 @@ def sys_info(fid=None, show_paths=False, *, dependencies='user'):
     use_mod_names = ('mne', 'numpy', 'scipy', 'matplotlib', '', 'sklearn',
                      'numba', 'nibabel', 'nilearn', 'dipy', 'cupy', 'pandas',
                      'mayavi', 'pyvista', 'pyvistaqt', 'ipyvtklink', 'vtk',
-                     'PyQt5', 'ipympl', 'mne_qt_browser')
+                     'PyQt5', 'ipympl', 'mne_qt_browser', 'pooch')
     if dependencies == 'developer':
         use_mod_names += (
             '', 'sphinx', 'sphinx_gallery', 'numpydoc', 'pydata_sphinx_theme',
